package com.fzfd.acg;

// import
import java.util.HashMap;
import java.util.Iterator;

import java.io.Reader;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.*;
import org.xml.sax.*;


/**
 * This class is the security access graph
 *
 *          NOTE: In a real system, one would want to use a special class for Roles vs. using
 *          strings as done here.
 *
 * @author Efraim Berkovich
 * @version 1.0
 */

public class Graph
{

  // XML constants -- For the security graph
  /** XML root node */
  static public final String XML_ROOT_NODE                       = "security";
  /** XML node for the list of roles */
  static public final String XML_ACCESS_ROLES_LIST               = "rolelist";
  /** XML node for defining a role */
  static public final String XML_ACCESS_ROLE                     = "role";
  /** XML node for the list of application flow nodes */
  static public final String XML_ACCESS_NODES_LIST               = "nodelist";
  /** XML node for the unique ID identifying an application flow node */
  static public final String XML_ACCESS_NODE                     = "node";
  /** XML node for the unique ID identifying an application flow node */
  static public final String XML_ACCESS_NODE_X                   = "x";
  /** XML node for the unique ID identifying an application flow node */
  static public final String XML_ACCESS_NODE_Y                   = "y";
  /** XML node for the list of allowed access paths */
  static public final String XML_ACCESS_PATH_LIST                = "accesslist";
  /** XML node for defining an access path */
  static public final String XML_ACCESS_PATH                     = "access";
  /** XML attribute for the start of an access path */
  static public final String XML_ACCESS_PATH_START_NODE          = "start";
  /** XML attribute for the end of an access path */
  static public final String XML_ACCESS_PATH_END_NODE            = "end";
  /** XML node for an allowed role to this path */
  static public final String XML_ACCESS_ALLOWED_ROLE             = "allowedrole";


  // instance variables
  private HashMap graph;

  /**
   * Make the empty graph
   */
  public Graph()
  {
    graph = new HashMap();
  }



  /**
   * Return a specified node
   * @param the nodeName
   * @return the node
   */
  public Node getNode( String nodeName )
  {
    return (Node)graph.get( nodeName );
  }

  /**
   * Get the nodes
   * @return the Iterator on the nodes
   */
  public Iterator getNodes()
  {
    return graph.values().iterator();
  }



  /**
   * Add a node. If graph contains one with this ID, it will not add.
   */
  public void addNode( Node node )
  {
    synchronized( graph )
    {
      if( ! graph.containsKey( node.getID() ) )
      {
        graph.put( node.getID(), node );
      }
    }
  }

  /**
   * Remove a node
   * @param node the node to remove
   */
  public void removeNode( Node node )
  {
    synchronized( graph )
    {
      // find and remove all incoming paths
      Iterator iter = graph.values().iterator();
      while( iter.hasNext() )
      {
        Node nd = (Node)iter.next();
        Vertex path = nd.getPathTo( node );
        if( path != null )
          nd.removeAccess( path );
      }

      graph.remove( node.getID() );
      node.removeAllAccess();
    }
  }


  /**
   * Remove a path
   * @param path the path to remove
   */
  public void removePath( Vertex path )
  {
    path.getEndNode().removeAccess( path );
  }



  /**
   * Add a path between two nodes. If path already exists, nothing is done.
   * If the nodes do not exist in the graph, nothing is done.
   * @param startNode Node which is the start of the path
   * @param endNode Node which is the end of the path
   * @return the new path
   */
  public Vertex addPath(  Node start
                        , Node end )
  {
    Node startNode = this.getNode( start.getID() );
    Node endNode = this.getNode( end.getID() );
    if( startNode == null )
      throw new NullPointerException( "Cannot find start node: " + start.getID() );
    if( endNode == null )
      throw new NullPointerException( "Cannot find end node: " + end.getID() );

    Vertex path = new Vertex( start, end );

    start.addAccess( path );
    return path;
  }


  /**
   * Add a role to an access path.
   * @param path The Vertex to which to add the role
   * @param role The Role to add
   */
  public void addRoleToPath( Vertex path, String role )
  {
    path.addAccess( role );
  }


  /**
   * Clear the graph
   */
  public void clear()
  {
    Node appflowNode;

    // clean-up memory from the graph -- this helps the GC
    Iterator iter = graph.values().iterator();
    while( iter.hasNext() )
    {
      appflowNode = (Node)iter.next();
      appflowNode.removeAllAccess();
    }

    // clear all
    graph.clear();
  }


  /**
   * Check for access
   */
  public boolean hasAccess( String[] roles, String source, String destination )
  {
    Node start = this.getNode( source );
    if( start == null ) throw new NullPointerException( "Cannot find source node: " + source );

    Node end = this.getNode( destination );
    if( start == null ) throw new NullPointerException( "Cannot find destination node: " + destination );

    for( int i = 0; i < roles.length; i++ )
    {
      if( start.hasAccess( roles[i], destination ) )
        return true;
    }
    return false;
  }


  /**
   * Read the security graph from a file and create a graph based on it
   * @param xmlReader the Reader which will provide the XML
   *
   * @exception java.io.IOException on problems reading xml source
   * @exception org.xml.sax.SAXException on XML issues
   */
  public void loadFromXML( Reader xmlReader )
              throws  java.io.IOException
              ,       org.xml.sax.SAXException
  {
    com.fzfd.acg.Node appflowNode;

    // open the file and parse the XML
    DOMParser parser = new DOMParser();
    parser.parse( new InputSource( xmlReader ) );
    Document xmlDoc = parser.getDocument();


    // parse the data
    NodeList list, accessList;
    org.w3c.dom.Node node, nd;
    NamedNodeMap attribs;

    HashMap securityRoles = new HashMap();

    // read all security roles
    list =  xmlDoc.getDocumentElement().getElementsByTagName( XML_ACCESS_ROLES_LIST );
    if( list.getLength() == 0 )
      throw new SAXException( "Cannot find node: " + XML_ACCESS_ROLES_LIST );
    list = list.item( 0 ).getChildNodes();

    for( int i = 0; i < list.getLength(); i++ )
    {
      node = list.item( i );
      if( node.getNodeName().equals( XML_ACCESS_ROLE ) )
      {
        node = node.getFirstChild();
        securityRoles.put( node.getNodeValue(), node.getNodeValue() );
      }
    }

    // read all nodes
    list =  xmlDoc.getDocumentElement().getElementsByTagName( XML_ACCESS_NODES_LIST );
    if( list.getLength() == 0 )
      throw new SAXException( "Cannot find node: " + XML_ACCESS_NODES_LIST );
    list = list.item( 0 ).getChildNodes();

    for( int i = 0; i < list.getLength(); i++ )
    {
      node = list.item( i );
      if( node.getNodeName().equals( XML_ACCESS_NODE ) )
      {
        node = node.getFirstChild();
        appflowNode = new com.fzfd.acg.Node( node.getNodeValue() );
        addNode( appflowNode );
      }
    }

    // read the access paths
    list =  xmlDoc.getDocumentElement().getElementsByTagName( XML_ACCESS_PATH_LIST );
    if( list.getLength() == 0 )
      throw new SAXException( "Cannot find node: " + XML_ACCESS_PATH_LIST );
    list = list.item( 0 ).getChildNodes();

    for( int i = 0; i < list.getLength(); i++ )
    {
      node = list.item( i );
      if( node.getNodeName().equals( XML_ACCESS_PATH ) )
      {
        attribs = node.getAttributes();

        node = attribs.getNamedItem( XML_ACCESS_PATH_START_NODE );
        com.fzfd.acg.Node start = getNode( node.getNodeValue() );
        if (start == null)
        {
          throw new SAXException("Security graph: startnode "  + node.getNodeValue() + " not found");
        }
        node = attribs.getNamedItem( XML_ACCESS_PATH_END_NODE );
        com.fzfd.acg.Node end = getNode( node.getNodeValue() );
        if (end == null)
        {
          throw new SAXException("Security graph: endnode "  + node.getNodeValue() + " not found");
        }
        Vertex path = addPath( start, end );

        accessList = list.item( i ).getChildNodes();
        for( int j = 0; j < accessList.getLength(); j++ )
        {
          node = accessList.item( j );
          if( node.getNodeName().equals( XML_ACCESS_ALLOWED_ROLE ) )
          {
            node = node.getFirstChild();
            String role = (String)securityRoles.get( node.getNodeValue() );
            if( role == null )
              throw new SAXException( "Cannot find security role: " + node.getNodeValue() );
            path.addAccess( role );
          }
        }
      }
    }
  }





}