package com.fzfd.acg;

// imports
import java.util.HashMap;
import java.util.Iterator;


/**
 * Defines a particular point within the application flow.
 * @author Efraim Berkovich
 * @version 1.0
 */
public class Node
{

  // instance variables
  private String ID;
  private HashMap outgoingPaths;


  /**
   * Create a node. Package view only
   *
   * @param ID unique identifier of the node
   */
  public Node( String ID )
  {
    outgoingPaths = new HashMap();
    this.ID = ID;
  }



  /**
   * Return the ID of the node
   * @return ID of the node
   */
  public String getID()
  {
    return ID;
  }


  /**
   * Get path specified by end node.
   * @return Vertex or null if not found
   */
  public Vertex getPathTo( Node dest )
  {
    return (Vertex)outgoingPaths.get( dest.getID() );
  }


  /**
   * Return the outgoing paths for this node
   * @return The outgoing paths for this node as an Iterator
   */
  public Iterator getPaths()
  {
    return this.outgoingPaths.values().iterator();
  }

  /**
   * Find whether role can go to the specified node from this node.
   * @param securityRole the Role asking for permission
   * @param destinationNode the String ID of the Node we want to go to
   * @return does this role have access?
   */
  public boolean hasAccess( String securityRole, String destinationNode )
  {
    // find the specified access path, keyed on destination node
    Vertex path = (Vertex)outgoingPaths.get( destinationNode );
    if( path == null )
      return false;

    return path.hasAccess( securityRole );
  }



  /**
   * Add an access path to this node. Package view only
   * @param path the Vertex to add
   */
  void addAccess( Vertex path )
  {
    if( outgoingPaths.containsKey( path.getEndNode().ID ) )
      return;

    if( path.getStartNode() != this )
      return;

    outgoingPaths.put( path.getEndNode().ID, path );
  }



  /**
   * Remove an access path from this node. Package view only
   * @param path the AccessPath to remove
   */
  void removeAccess( Vertex path )
  {
    if( outgoingPaths.containsKey( path.getEndNode().ID ) )
    {
      outgoingPaths.remove( path.getEndNode().ID );
    }
  }



  /**
   * Remove all access paths. Package view only
   */
  void removeAllAccess( )
  {
    outgoingPaths.clear();
  }

}





