package com.fzfd.acg;

// imports
import java.util.HashMap;
import java.util.Iterator;

/**
 * Determines a path between two Nodes and provides
 *               the restrictions on who may follow this path.
 *
 *          NOTE: In a real system, one would want to use a special class for Roles vs. using
 *          strings as done here.
 *
 * @author Efraim Berkovich
 * @version 1.0
 */
public class Vertex
{

  // instance variables
  private HashMap permittedRoles;       // security roles which can use this path
  private HashMap permittedResponses;   // responses from the startNode which should use this path

  private Node endNode;
  private Node startNode;

  /**
   * Create the object. Package visible only
   * @param startNode beginning of path
   * @param endNode end of path
   */
  public Vertex( Node startNode, Node endNode )
  {
    permittedRoles = new HashMap();
    this.startNode = startNode;
    this.endNode = endNode;
  }


  /**
   * Return the end node of this path
   * @return the end node of this path
   */
  public Node getEndNode()
  {
    return endNode;
  }



  /**
   * Return the start node of this path
   * @return the start node of this path
   */
  public Node getStartNode()
  {
    return startNode;
  }



  /**
   * Find if a particular role has access
   * @param securityRole the role we want to check
   * @return does this role have access?
   */
  public boolean hasAccess( String securityRole )
  {
    return permittedRoles.containsKey( securityRole );
  }



  /**
   * Add a particular access role. Package visible only. NOT synchronized since
   * we don't expect to do this except at init.
   */
  void addAccess( String securityRole )
  {
    permittedRoles.put( securityRole, securityRole );
  }



  /**
   * Remove an access role. Package visible only. NOT synchronized since
   * we don't expect to do this except at init.
   */
  void removeAccess( String securityRole )
  {
    permittedRoles.remove( securityRole );
  }



  /**
   * Get all roles as string[].
   * @return the role names as an array
   */
  public String[] getRoles()
  {
    String[] out = new String[ permittedRoles.size() ];
    Iterator iter = permittedRoles.values().iterator();
    for( int i = 0; i < out.length; i++ )
    {
      String role = (String)iter.next();
      out[i] = role;
    }
    return out;
  }

}







