package com.fzfd.web;

import com.fzfd.acg.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.IOException;

/**
 * Controller handles routing HttpRequests.
 *
 * @author Efraim Berkovich
 * @version 1.4
 */

public class ControllerServlet extends HttpServlet
{

  protected ServletConfig servletConfig                   = null;


  /** Initialize */
  public void init( ServletConfig config ) throws ServletException
  {
    super.init( config );
    servletConfig = config;
  }



  /** Process the HTTP Get request */
  public void doGet(HttpServletRequest request, HttpServletResponse response)
                        throws ServletException, IOException
  {
    executeRequest( request, response );
  }


  /** Process the HTTP Post request */
  public void doPost(HttpServletRequest request, HttpServletResponse response)
          throws ServletException, IOException
  {
    executeRequest( request, response );
  }


  /**
   * Clean up
   */
  public void destroy()
  {
    servletConfig = null;
  }


  /**
   * This is the method which actually handles all the web requests.
   * The servlet passes the response obtained by the  method execution to the forwarding JSP.
   */
  protected void executeRequest(   HttpServletRequest request
                               ,   HttpServletResponse response )
                        throws ServletException,IOException
  {

    String forwardingURL = handleRequest( request, response );

    RequestDispatcher rd =  servletConfig.getServletContext().getRequestDispatcher( forwardingURL );
    if( rd != null )
    {
      System.out.println( "Forwarding to URL: " + forwardingURL );
      rd.forward( request, response );
    }
    else
    {
      String message = "Unable to get the request dispatcher for the JSP: " + forwardingURL;
      System.err.println( message );
      throw new ServletException( message );
    }

  } // executeRequest


  /**
   * Checks for a valid session and routes the request.
   * @return the URL to route the request to
   */
  protected String handleRequest(   HttpServletRequest request
                                  , HttpServletResponse response )
  {
    String forwardingURL = null;

    try
    {
      HttpSession session = request.getSession( false );

      if( ( session == null ) )
      {
        return handleLoggedOut( request );
      }

      String source = (String)session.getAttribute( RoutingMap.SERVICE_SOURCE );
      String destination = request.getParameter( RoutingMap.SERVICE_REQUESTED );
      String[] roles = (String[])session.getAttribute( RoutingMap.USER_ROLES );

      if( roles == null )
      {
        return handleLoggedOut( request );
      }

      if( ACG.hasAccess( roles, source, destination ) )
      {
        forwardingURL = handleRegular( request );
        session.setAttribute( RoutingMap.SERVICE_SOURCE, destination );
      }
      else
      {
        forwardingURL = handleUnauthorized( request );
      }
   }
    catch( Exception e )
    {
      // Log exception
      e.printStackTrace();

      forwardingURL = handleError( e, request );
    }
    return forwardingURL;
  }



  /**
   * Handle an exception
   * @param exception Exception
   * @param req The request
   * @param request HttpServletRequest
   * @param response HttpServletResponse
   */
  private String handleError(   Exception           exception
                             ,  HttpServletRequest  request )
  {
    Message out = new Message( );
    out.setProperty( "EXCEPTION", exception );

    request.setAttribute( RoutingMap.SERVICE_RESPONSE, out );

    // forward to error page
    return routingMap.getErrorPage();
  }




  /**
   * Call the service and find the re-direct
   * @param aimsReq The request
   * @param req HttpServletRequest
   * @param res HttpServletResponse
   */
  private String handleRegular( HttpServletRequest  request )
                                throws Exception
  {

    // NOTE: This is for illustration only. One would want to use a more
    //   data-driven design.

    String destination = request.getParameter( RoutingMap.SERVICE_REQUESTED );
    String forwardingURL = routingMap.findForwardingURL( destination );
    Message out = null;
    Message in = new Message( request );

    if( destination.equals( RoutingMap.SERVICE_CHANGE_PASSWORD ) )
    {
      out = MainFacade.getInstance().changePassword( in );
    }
    if( destination.equals( RoutingMap.SERVICE_EDIT ) )
    {
      out = MainFacade.getInstance().editRecords( in );
    }
    if( destination.equals( RoutingMap.SERVICE_LOGS ) )
    {
      out = MainFacade.getInstance().getLogs( in );
    }
    if( destination.equals( RoutingMap.SERVICE_VIEW ) )
    {
      out = MainFacade.getInstance().viewRecords( in );
    }
    if( destination.equals( RoutingMap.SERVICE_MAIN ) )
    {
      out = MainFacade.getInstance().viewMain( in );
    }

    if( out == null )
      throw new RuntimeException( "Unable to find service: " + destination );

    request.setAttribute( RoutingMap.SERVICE_RESPONSE, out );
    return forwardingURL;
  }






  /**
   * Special request handling on not authorized event
   * @param req The request
   * @param request HttpServletRequest
   * @param response HttpServletResponse
   */
  private String handleUnauthorized( HttpServletRequest  request )
  {
    return routingMap.getNotAuthorizedPage();
  }


  /**
   * Request handling for invalid session
   */
  private String handleLoggedOut( HttpServletRequest request )
  {
    return routingMap.getLogoutPage();
  }

  // static
  static private RoutingMap routingMap        = null;
  static private Graph ACG                    = null;

  /**
   * Initialize the controller.
   * @param map RoutingMap to use for routing.
   */
  static public void initialize( RoutingMap map, Graph graph )
  {
    routingMap = map;
    ACG = graph;
  }

  /**
   * REturn the routing map for this servlet
   */
  static public RoutingMap getRoutingMap()
  {
    return routingMap;
  }



}