package com.fzfd.web;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.IOException;


/**
 * Handles making the session for the user. Then hands off to controller.
 *
 * @author Efraim Berkovich
 * @version 1.4
 */

public class LoginServlet extends HttpServlet
{

  /** Process the HTTP Get request */
  public void doGet(HttpServletRequest request, HttpServletResponse response)
                        throws ServletException, IOException
  {
    executeRequest( request, response );
  }


  /** Process the HTTP Post request */
  public void doPost(HttpServletRequest request, HttpServletResponse response)
          throws ServletException, IOException
  {
    executeRequest( request, response );
  }


    /**
   * This is the method which actually handles all the web requests.
   * The servlet passes the response obtained by the  method execution to the forwarding JSP.
   */
  protected void executeRequest(   HttpServletRequest request
                               ,   HttpServletResponse response )
                        throws ServletException,IOException
  {

    String forwardingURL = handleRequest( request, response );

    RequestDispatcher rd =  servletConfig.getServletContext().getRequestDispatcher( forwardingURL );
    if( rd != null )
    {
      System.out.println( "Forwarding to URL: " + forwardingURL );
      rd.forward( request, response );
    }
    else
    {
      String message = "Unable to get the request dispatcher for the JSP: " + forwardingURL;
      System.err.println( message );
      throw new ServletException( message );
    }

  } // executeRequest



  protected ServletConfig servletConfig                   = null;


  /** Initialize */
  public void init( ServletConfig config ) throws ServletException
  {
    super.init( config );
    servletConfig = config;
    try
    {
      String securityXMLfile = config.getInitParameter( "securityFileLocation" );

      // note this code is for illustration only. should be done as a data-driven design
      int logoutInterval = 30;
      String errorPage = "/error/debug_error.jsp";
      String notAuthorizedPage = "/error/notAuthorized.jsp";
      String logoutPage = "/logout.html";
      String controllerURL = "/c";
      String loginURL = "/login";
      String loginPage = "/index.jsp";

      RoutingMap map = new RoutingMap( loginPage, errorPage, notAuthorizedPage, logoutPage, controllerURL, loginURL, logoutInterval );

      com.fzfd.acg.Graph graph = new com.fzfd.acg.Graph();
      graph.loadFromXML( new java.io.FileReader( securityXMLfile ) );

      ControllerServlet.initialize( map, graph );
      LoginServlet.initialize( map );

      System.out.println( "Completed initialization" );
    }
    catch( Exception e )
    {
      System.err.println( "STARTUP ERROR: " );
      System.err.println( e );
    }
  }



  /**
   * Login user, create session.
   * @return The URL to forward the http requests to
   */
  protected String handleRequest(  HttpServletRequest request, HttpServletResponse response )
  {
    String forwardingURL = null;

    try
    {
      InLoginMessage req = new InLoginMessage( request );
      OutLoginMessage out = MainFacade.getInstance().login( req );

      if( out.getSuccess() )
      {
        forwardingURL = routingMap.findForwardingURL( RoutingMap.SERVICE_LOGIN );
        HttpSession session = request.getSession( true );
        session.setMaxInactiveInterval( routingMap.getSessionTimeOut() );
        session.setAttribute( RoutingMap.USER_ROLES, out.getRoles() );
        session.setAttribute( RoutingMap.SERVICE_SOURCE, RoutingMap.SERVICE_MAIN );

        // just for convenience keep the last user id in cookie
        response.addCookie( new Cookie( InLoginMessage.FIELD_USER_ID, req.getUserID() ) );
      }
      else
      {
        forwardingURL = routingMap.getLoginPage();
      }

      request.setAttribute( RoutingMap.SERVICE_RESPONSE, out );
    }
    catch( Exception e )
    {
      // Log exception
      e.printStackTrace();
      forwardingURL = handleError( e , request );
    }

    return forwardingURL;
  }


 /**
   * Handle an exception
   * @param exception Exception
   * @param req The request
   * @param request HttpServletRequest
   * @param response HttpServletResponse
   */
  private String handleError(   Exception           exception
                             ,  HttpServletRequest  request )
  {
    Message out = new Message( );
    out.setProperty( "EXCEPTION", exception );

    request.setAttribute( RoutingMap.SERVICE_RESPONSE, out );

    // forward to error page
    return routingMap.getErrorPage();
  }






  // static
  static private RoutingMap routingMap        = null;

  /**
   * Initialize the controller.
   * @param map RoutingMap to use for routing.
   */
  static public void initialize( RoutingMap map )
  {
    routingMap = map;
  }

  /**
   * REturn the routing map for this servlet
   */
  static public RoutingMap getRoutingMap()
  {
    return routingMap;
  }


}